{{
  regex_BBRdemo.spin

  test/demo program originally written by Phil Pilgrim to demonstrate his spin-based
  "regular expression" parser.

  the example for use was parsing an NMEA $GPRMC sentence and extracting long and lat.

  rewritten by Brian Riley, adding a second sentemce $GPGGA and to extract additional
  data (date and time (GPRMC and GPGGA), GPS fix quality, number of satellites, and
  altitude (GPGGA)

  brianbr@wulfden.org

}}


CON

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000

OBJ

  re    : "regex"
  io    : "FullDuplexSerial"

PUB  Start | teststr1, teststr2, GPRMCpat, GPGGApat

  io.start(31, 30, 0, 115200)
  io.tx(0)
  
  teststr1 := string("$GPGSV,2,1,08,01,40,083,46,02,17,308,41,12,07,344,39,14,22,228,45*75", 13, {
                 }  "$GPRMC,123519,A,4807.038,N,01131.000,E,022.4,084.4,230394,003.1,W*6A", 13, {
                 }  "$GPVTG,054.7,T,034.4,M,005.5,N,010.2,K*48", 13)

  teststr2 := string("$GPGSV,2,1,08,01,40,083,46,02,17,308,41,12,07,344,39,14,22,228,45*75", 13, {
                 }  "$GPGGA,170834,4124.8963,N,08151.6838,W,1,05,1.5,280.2,M,-34.0,M,,,*75", 13, {
                 }  "$GPVTG,054.7,T,034.4,M,005.5,N,010.2,K*48", 13)

  GPRMCpat := string("\$GPRMC\s*,($1\d+)\s*,[^,]*,($2[\d\.]+)\s*,($3N|S)\s*,($4[\d\.]+)\s*,", {
                 }   "($5E|W)\s*,[^,]*,[^,]*,($6\d+)")

  GPGGApat := string("\$GPGGA\s*,($1\d+)\s*,($2[\d\.]+)\s*,($3N|S)\s*,($4[\d\.]+)\s*,", {
                 }   "($5E|W)\s*,($6\d)\s*,($7\d+)\s*,[^,]*,($8[\d\.]+)\s*,($9M)")

  testparse(teststr1, GPRMCpat)

  testparse(teststr2, GPGGApat)

  repeat

PRI testparse(teststring, pattern) | resaddr, rslt, i

  io.str(string(13, "------------------", 13, "String:", 13, 13))
  io.str(teststring)
  io.str(string(13, 13, "Pattern:", 13))
  io.str(pattern)

  rslt := -cnt
  resaddr := re.match(teststring, pattern, re#NOALT)
  rslt += cnt

  io.str(string(13, 13, "Time: "))
  io.dec(rslt / 80_000)
  io.str(string(" ms."))
  io.str(string(13, 13, "Results:", 13))

  if (resaddr < 0)
    io.tx(13)
    io.str(string("Error #"))
    io.dec(-resaddr)
  elseif (resaddr == 0)
    io.tx(13)
    io.str(string("No match."))
  else
    repeat i from 0 to 9
      if (rslt := long[resaddr][i])
        io.tx(13)
        io.dec(i)
        io.str(string(": "))
        io.str(re.field(i))

    io.str(string(13, 13, "Remainder of string:", 13, 13))
    io.str(re.remainder)

